/*
 * SimAnnealingStepSolver.java
 *
 * Created on 9. nor 2006, 20:58
 *
 */

package savi.alg;

/**
 * Instance of SimAnnealingSkeleton, basic class for solving with simulated 
 * annealing algorithm. Solving can be stepped.
 * @author Premysl JIRIK
 */
public class SimAnnealingStepSolver extends SimAnnealingBasicSolver {
    /**
     * State type enumeration, currently imlemented are EQUILIBRIUM and COOL.
     */
    protected enum Steps { 
        /**
         * This state means, that algorithm is in the equlibrium loop.
         */
        EQUILIBRUM, 
        /**
         * This state means, that algorithm counting next temperature.
         */
        COOL };
    /**
     * State variable
     */
    protected Steps           nextStep;
    
    /**
     * Creates a new instance of SimAnnealingStepSolver, initialize startup variables.
     * @param s Initial state
     * @param temp Starting temperature value
     * @param finalTemp Final temperature value
     * @param coolFactor Cooling factor
     */
    public SimAnnealingStepSolver(StateSkeleton s, double temp, double finalTemp, double coolFactor, int equilibSteps) {
        super(s, temp, finalTemp, coolFactor, equilibSteps);
        nextStep = Steps.EQUILIBRUM;
    }
    
    /**
     * This method makes one step of SA algorithm.
     */
    public void makeStep() {
        if (!super.frozen()) {
            if (nextStep == Steps.EQUILIBRUM) {
                equilibrium();
                nextStep = Steps.COOL;
                return;
            }
            if (nextStep == Steps.COOL) {
                cool();
                nextStep = Steps.EQUILIBRUM;
                return;
            }
        }
    }
    
    /**
     * Used for end recognition.
     * @return returns true if algorthm finished, else return false.
     */
    public boolean isEnd() {
        return super.frozen();
    }
}
