/*
 * StateSkeleton.java
 *
 * Created on 12. prosinec 2005, 14:01
 *
 */

package savi.alg;

/**
 * Basic interface for states. All states used with SimAnnealingSkeleton 
 * (and derived classes) must implement this interface.
 * @author Premysl JIRIK
 */
public interface StateSkeleton extends Cloneable {
    /**
     * This function is called if algorithm finding initial state.
     * @param temp Starting temperature
     */
    void findInitialState(double temp);
    /**
     * Called when algorithm finding next state.
     * @param temp Actual temperature
     */
    void findNextState(double temp);
    /**
     * This function count optimalization criterion for the state.
     * @return Optimalization criterion
     */
    double getActualCost();
    
    /**
     * Creates and returns a copy of this object. The copy must be Deep!
     * @return cloned object of this class
     */
    Object clone();
    /**
     * This method return submission (problem instance) in text format. May return array
     * of lines with submission.
     * @return text lines represented actual problem instance
     */
    String [] getSubmission();
}
