/*
 * Items.java
 *
 * Created on 12. nor 2006, 13:09
 *
 */

package savi.alg.knapsack;

/**
 * Class representing knapsack problem instance.
 * @author Premysl JIRIK
 */
public class Items {
    /**
     * number of items in this knapsack instance
     */
    protected int count;
    /**
     * internal iterator for addItem method
     */
    protected int idx = 0;
    /**
     * array with items; one item is pair of integers (int[2]) - weight and cost
     */
    protected double items[][];
    
    /**
     * Creates a new instance of Items, creates empty array
     * @param count maximum items
     */
    public Items(int count) {
        this.count = count;
        items = new double[count][2];
    }
    /**
     * Creates a new instance of Items, set items
     * @param it initialized items
     */
    public Items(double [][] it) {
        setItems(it);
    }
    
    /**
     * Set prepared items
     * @param it items
     */
    public void setItems(double [][]it) {
        items = it;
        count = items.length;
    }
    
    /**
     * Insert new item into the instance
     * @param w item weight
     * @param c item cost
     */
    protected void addItem(double w, double c) {
        if (idx < count) {
            items[idx][0] = w;
            items[idx][1] = c;
            idx++;
        }
    }
    /**
     * Get current items count in knapsack instance
     * @return number of items
     */
    public int getCount() {
        return count;
    }
    /**
     * Randomly cretes items.
     * @param wmax maximum item weight
     * @param cmax maximum item cost
     */
    public void randomItems(double wmax, double cmax) {
        idx = 0;
        for (int i = 0; i < count; i++)
            addItem(wmax * Math.random(), cmax * Math.random());
    }
    /**
     * Return weight of desired item.
     * @param item item index
     * @return item weight
     */
    public double getItemWeight(int item) {
        return items[item][0];
    }
    /**
     * Return cost of desired item.
     * @param item item index
     * @return item cost
     */
    public double getItemCost(int item) {
        return items[item][1];
    }    
}
