/*
 * StateKnapsack.java
 *
 * Created on 12. nor 2006, 13:14
 *
 */

package savi.alg.knapsack;

import java.util.*;
import savi.alg.*;

/**
 * State representing knapsack problem.
 * @author Kip
 */
public class StateKnapsack implements StateSkeleton {
    /**
     * reference to available items
     */
    protected Items     items;
    /**
     * current cost
     */
    protected double    currCost;
    /**
     * current weight
     */
    protected double    currWeight;
    /**
     * capacity (maximum knapsack weight)
     */
    protected double    maxWeight;
    /**
     * number of items in knapsack
     */
    protected int       itmCount;
    /**
     * represent items, if they in knapsack
     */
    protected boolean   sack[];
    /**
     * random generator
     */
    protected Random    rnd;
    
    /**
     * Creates a new instance of StateKnapsack
     * @param items items list
     * @param maxWeight knapsack capacity
     */
    public StateKnapsack(Items items, double maxWeight) {
        this.items = items;
        this.maxWeight = maxWeight;
        sack = new boolean[items.getCount()];
        rnd = new Random();
    }
        
    /**
     * Initialize state
     * @param temp starting temperature
     */
    public void findInitialState(double temp) {
        currCost = 0;
        currWeight = 0;
        itmCount = 0;
        for (int i = 0; i < items.getCount(); i++) {
            if (currWeight + items.getItemWeight(i) < maxWeight) {
                sack[i] = true;
                currWeight += items.getItemWeight(i);
                currCost += items.getItemCost(i);
                itmCount++;
            } else
                sack[i] = false;
        }
        
        // all items in knapsack
    }

    /**
     * Finds next state of knapsack problem.
     * @param temp current temperature
     */
    public void findNextState(double temp) {
        int pos = rnd.nextInt(items.getCount());
        
        if (sack[pos]) {
            sack[pos] = false;
            currWeight -= items.getItemWeight(pos);
            currCost -= items.getItemCost(pos);
            itmCount--;
        } else {
            if (currWeight + items.getItemWeight(pos) <= maxWeight) {
                sack[pos] = true;
                currWeight += items.getItemWeight(pos);
                currCost += items.getItemCost(pos);
                itmCount++;
            }
        }
        
/*        if (rnd.nextInt() % 2 == 0) {                    // change
            int chgA = rnd.nextInt(sack.length);
            int chgB = rnd.nextInt(sack.length);      
            while (sack[chgA] == sack[chgB])
                chgB = rnd.nextInt(sack.length);
            if (sack[chgA]) {                       // swap? (we want chgA not in sack)
                int tmp = chgA; chgA = chgB; chgB = tmp;
            }
            if (currWeight + items.getItemWeight(chgA) - items.getItemWeight(chgB) <= maxWeight) {
                sack[chgA] = true;
                sack[chgB] = false;
                currWeight += items.getItemWeight(chgA) - items.getItemWeight(chgB);
                currCost += items.getItemCost(chgA) - items.getItemCost(chgB);
            }
        } else {                                        
            if ((rnd.nextInt() % 1 == 0) || (itmCount == 0)) {               // add
                int add = rnd.nextInt(sack.length);
                while (sack[add])
                    add = rnd.nextInt(sack.length);
                if (currWeight + items.getItemWeight(add) <= maxWeight) {
                    sack[add] = true;
                    currWeight += items.getItemWeight(add);
                    currCost += items.getItemCost(add);
                    itmCount++;
                }
            } else {                                    // remove  
                int rem = rnd.nextInt(sack.length);
                while (!sack[rem])
                    rem = rnd.nextInt(sack.length);
                sack[rem] = false;
                currWeight -= items.getItemWeight(rem);
                currCost -= items.getItemCost(rem);
                itmCount--;
            }
        }
 */
    }
    /**
     * This method count current cost.
     * @return actual cost
     */
    public double getRealCost() {
        return currCost;
    }
    /**
     * Count actual optimalization criterion.
     * @return value of optimalization criterion
     */
    public double getActualCost() {
        return 1.0 / currCost;
    }
    /**
     * Creates and returns a copy of this object.
     * @return deep copy of this object
     */
    public Object clone() {
        StateKnapsack copy = null;
        try {   
            copy = (StateKnapsack)super.clone();
            copy.sack = new boolean[sack.length];
            for (int i = 0; i < sack.length; i++)
                copy.sack[i] = sack[i];
        }
        catch (CloneNotSupportedException e) {
            e.printStackTrace();
        }
        return copy;
    }
    /**
     * Recount current cost.
     * @return actual cost
     */
    public double recountCost() {
        double c = 0;
        for (int i = 0; i < sack.length; i++) 
            if (sack[i])
                c += items.getItemCost(i);
        return c;
    }
    /**
     * Get count of items in knapsack.
     * @return count of items in knapsack
     */
    public int getSelectedItemsCount() {
        return itmCount;
    }
    /**
     * Count weight of items in knapsack.
     * @return weight
     */
    public double getWeight() {
        return currWeight;
    }
    /**
     * Return true/false list of items representing content of knapsack
     * @return true/false list of items
     */
    public boolean[] getKnapsack() {
        return sack;
    }
    /**
     * Return available items.
     * @return items
     */
    public Items getItems() {
        return items;
    }    
    /**
     * Set maximum weight of items in knapsack
     * @param capacity capacity
     */
    public void setCapacity(double capacity) {
        maxWeight = capacity;
    }
    /**
     * Set available items.
     * @param items instance of class Items
     */
    public void setItems(Items items) {
        this.items = items;
        sack = new boolean[items.getCount()];
        findInitialState(0);
    }

    /**
     * Return text representation of knapsack instance.
     * @return array of lines
     */
    public String [] getSubmission() {
        String line = "";
        
        line = "1 " + Integer.toString(items.count) + " " + Integer.toString((int)maxWeight);
        line += " ";
        for (int i = 0; i < items.count; i++) {
            line += Integer.toString((int)(items.getItemWeight(i))) + " " +
                    Integer.toString((int)(items.getItemCost(i))) + " ";
        } 
        
        String [] fld = new String[1];
        fld[0] = line;
        return fld;
    }
}
