/*
 * Clause.java
 *
 * Created on 2. erven 2006, 23:21
 *
 */

package savi.alg.sat3;

import java.util.Vector;

/**
 * Object representing a clause.
 * @author Kip
 */
public class Clause {
    /**
     * list of variables index
     */
    protected int varsIndex[];
    /**
     * array of negation for variables
     */
    protected boolean varsNegation[];
    
    /**
     * Creates a new instance of Clause, generate random clause from parameters.
     * @param varsCount number of Clauses
     * @param clauseLen number of literals in clause
     */
    public Clause(int varsCount, int clauseLen) {
        varsIndex = new int[clauseLen];
        varsNegation = new boolean[clauseLen];
        
        for (int i = 0; i < clauseLen; i++) {
            varsIndex[i] = (int)(Math.random() * varsCount);
            varsNegation[i] = Math.random() > 0.5;
        }
    }
    /**
     * Creates a new instance of Clause from one dimacs format line.
     * see http:\\www.satlib.org
     * @param dimacsClause vector with numbers representing clause
     */
    public Clause(Vector dimacsClause) {
        varsIndex = new int[dimacsClause.size()];
        varsNegation = new boolean[dimacsClause.size()];
        
        for (int i = 0; i < dimacsClause.size(); i++) {
            varsIndex[i] = Math.abs((Integer)(dimacsClause.elementAt(i))) - 1;
            varsNegation[i] = (Integer)(dimacsClause.elementAt(i)) < 0;
        }
    }
    
    /**
     * Return true if the clause is satisfied vith given variables assignment.
     * @param vars variables assignment
     * @return true if clause is satisfact
     */
    public boolean isSatisfact(boolean vars[]) {
        for (int i = 0; i < varsIndex.length; i++) {
            if (vars[varsIndex[i]] ^ varsNegation[i])
                return true;
        }
        return false;
    }
    
    /**
     * Return number of literals in clause.
     * @return number of literals
     */
    public int getLiteralCount() {
        return varsIndex.length;
    }
    /**
     *  Return text representation of the clause. This represenatation is printed to user.
     * @return text representation
     */
    public String getLiteralStringRepresentation() {
        String s = "(";
        for (int i = 0; i < varsIndex.length; i++) {
            if (varsNegation[i])
                s += "!";
            else 
                s += " ";
            s += "x[" + String.format("%02d", varsIndex[i]) + "]";
            if (i < varsIndex.length - 1)
                s += " . ";
        }
        s += ")";
        return s;
    }
    
    /**
     *  Return dimacs text representation of the clause.
     * @return text represenation of the clause.
     */
    public String getCnfRepresentation() {
        String s = "";
        for (int i = 0; i < varsIndex.length; i++) {
            if (varsNegation[i])
                s += "-";
            s += Integer.toString(varsIndex[i] + 1);
            s += " ";
        }
        s += "0";
        return s;        
    }
}
