/*
 * ViewKnapsack.java
 *
 * Created on 12. nor 2006, 15:52
 *
 */

package savi.alg.weighsat3;

import java.awt.*;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.text.DecimalFormat;
import savi.alg.*;
import savi.visualization.support.VisualizationPanel;
import savi.alg.sat3.Clause;
import savi.alg.sat3.Formula;

/**
 * Visualization panel for MAX-WEIGHTED-3-SAT problem.
 * @author Premysl JIRIK
 */
public class ViewWeightedSAT3 extends VisualizationPanel {
    Formula                 formula;
//    List                    lstItems;
    
    ScrollPane              pane;
    WSatListCanvas          draw;
    ScrollPane              varsPane;
    WSatVariableListCanvas  varsDraw;
    
    /**
     * Creates a new instance of Viewweighted3SAT.
     */
    public ViewWeightedSAT3() {
        super();
        initScreen();
    }
    /**
     * Creates a new instance of Viewweighted3SAT and initialize state.
     * @param s state
     */
    public ViewWeightedSAT3(StateSkeleton s) {
        setState(s);
        initScreen();
    }
    
    /**
     * Initialize draw of items.
     */
    protected void initScreen() {
        setBackground(Color.WHITE);
        setLayout(null);
        
        pane = new ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);
        pane.setBounds(5, 25, 260, 415);
        add(pane);
        pane.setWheelScrollingEnabled(true);
        pane.getVAdjustable().setUnitIncrement(16);
        Label lb = new Label("Clauses");
        lb.setBounds(5, 5, 100, 20);
        add(lb);
        
        draw = new WSatListCanvas(this);
        draw.setSize(new Dimension(200, 200));
        pane.add(draw);

        varsPane = new ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);
        varsPane.setBounds(270, 25, 140, 415);
        add(varsPane);
        varsPane.setWheelScrollingEnabled(true);
        varsPane.getVAdjustable().setUnitIncrement(16);
        lb = new Label("Variables (weights)");
        lb.setBounds(270, 5, 125, 20);
        add(lb);

        varsDraw = new WSatVariableListCanvas(this);
        varsDraw.setSize(new Dimension(200, 200));
        varsPane.add(varsDraw);
        
        validate();
        setVisible(true);
    }
    
    /**
     * Updates scene (repaint visualization)
     */
    public void updateScene() {
        if (formula != ((StateWSAT3)state).getFormula())
            formula = ((StateWSAT3)state).getFormula();
        draw.refresh();
        varsDraw.refresh();
    }

    class WSatListCanvas extends Canvas {
        VisualizationPanel  vp = null;
        int                 imageHeight = -1;
        Image               buffImage;
        Graphics            offscreen;
        boolean             initDone = false;

        WSatListCanvas(VisualizationPanel vp) {
            super();
            this.vp = vp;
            setBackground(Color.WHITE);
        }
        
        public void paint(Graphics g) {
            if (!initDone)
                initpaint(g);
            g.drawImage(buffImage, 0, 0, this);
            pane.validate();
        }

        public void update(Graphics g) {
            if (!initDone)
                initpaint(g);
            g.drawImage(buffImage, 0, 0, this);
        }
        
        protected int desiredHeight() {
            return formula.clausesCount() * 16 + 10;
        }
        
        public void refresh() {
            if (desiredHeight() != imageHeight) {
                this.setSize(225, desiredHeight());
                this.validate();
            }
            initDone = false;
            repaint();
        }
        
        protected void initpaint(Graphics g) {
            try {
                if (imageHeight != desiredHeight()) {
                    buffImage = this.createImage(225, desiredHeight());
                    offscreen = buffImage.getGraphics();
                    imageHeight = desiredHeight();
                }
                offscreen.setColor(Color.WHITE);
                offscreen.fillRect(0, 0, 225, imageHeight);
                
                Clause[] clauses = ((StateWSAT3)state).getFormula().getClauses();
                boolean [] vars = ((StateWSAT3)state).getVariableValues();
                
                offscreen.setFont(new Font("Monospaced", 0, 11));
                for (int i = 0; i < clauses.length; i++) {

                    if (clauses[i].isSatisfact(vars))
                        offscreen.setColor(Color.GREEN);
                    else
                        offscreen.setColor(Color.PINK);
                    offscreen.fillRect(0, i * 16, 225, (i + 1) * 16);

                    offscreen.setColor(Color.WHITE);
                    offscreen.drawRect(0, i * 16, 225, (i + 1) * 16);
    
                    offscreen.setColor(Color.BLACK);
                    offscreen.drawString(clauses[i].getLiteralStringRepresentation(), 10, i * 16 + 12);
                }
                initDone = true;
            }
            catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    class WSatVariableListCanvas extends Canvas {
        VisualizationPanel  vp = null;
        int                 imageHeight = -1;
        Image               buffImage;
        Graphics            offscreen;
        boolean             initDone = false;

        WSatVariableListCanvas(VisualizationPanel vp) {
            super();
            this.vp = vp;
            setBackground(Color.WHITE);
        }
        
        public void paint(Graphics g) {
            if (!initDone)
                initpaint(g);
            g.drawImage(buffImage, 0, 0, this);
            varsPane.validate();
        }

        public void update(Graphics g) {
            if (!initDone)
                initpaint(g);
            g.drawImage(buffImage, 0, 0, this);
        }
        
        protected int desiredHeight() {
            return formula.varsCount() * 16 + 10;
        }
        
        public void refresh() {
            if (imageHeight != desiredHeight()) {
                this.setSize(115, desiredHeight());
                this.validate();
            }
            initDone = false;
            repaint();
        }
        
        protected void initpaint(Graphics g) {
            try {
                if (imageHeight != desiredHeight()) {
                    buffImage = this.createImage(115, desiredHeight());
                    offscreen = buffImage.getGraphics();
                    imageHeight = desiredHeight();
                }
                offscreen.setColor(Color.WHITE);
                offscreen.fillRect(0, 0, 115, imageHeight);
                
                boolean[] vars = ((StateWSAT3)state).getVariableValues();
                int[] weights = ((StateWSAT3)state).getFormula().getWeights();
                
                offscreen.setFont(new Font("Monospaced", 0, 12));
                for (int i = 0; i < vars.length; i++) {

                    if (vars[i])
                        offscreen.setColor(Color.GREEN);
                    else
                        offscreen.setColor(Color.PINK);
                    offscreen.fillRect(0, i * 16, 115, (i + 1) * 16);

                    offscreen.setColor(Color.WHITE);
                    offscreen.drawRect(0, i * 16, 115, (i + 1) * 16);
    
                    offscreen.setColor(Color.BLACK);
                    String s;
                    if (vars[i])
                        s = "x[" + String.format("%02d", i) + "]; ";
                    else
                        s = "x[" + String.format("%02d", i) + "]; ";
                    s += String.format("w: %d", weights[i]);
                    offscreen.drawString(s, 10, i * 16 + 13);
                }
                initDone = true;
            }
            catch (Exception e) {
                e.printStackTrace();
            }
        }
    }
}

/*
            lstItems.removeAll();
 
            DecimalFormat idx = new DecimalFormat("0");
            DecimalFormat num = new DecimalFormat("0");
                lstItems.add("Item #" + idx.format(i) + " - Weight: " 
                        + num.format(itmList.getItemWeight(i)) + "     Cost: "
                        + num.format(itmList.getItemWeight(i)));

            String fmtNum = "%04d";
            String fmtIdx = "%03d";
            for (int i = 0; i < itmList.getCount(); i++)
                lstItems.add("Item #" + String.format(fmtIdx, i) + " - Weight: " 
                        + String.format(fmtNum, (int)itmList.getItemWeight(i)) + "     Cost: "
                        + String.format(fmtNum, (int)itmList.getItemCost(i)));
        }
        boolean []sack = ((StateKnapsack)alg.getState()).getKnapsack();
        for (int i = 0; i < itmList.getCount(); i++) {
            if (sack[i]) 
                lstItems.select(i);
            else
                lstItems.deselect(i);
        }
        lstItems.makeVisible(0);
*/

/*
    
class myCanvas extends Canvas {
  Image    buffImage;
  Graphics offscreen;
  boolean initDone = false;

  myCanvas() { super(); }
    public void paint(Graphics g) {
   if (!initDone)
      initpaint(g);
   else
      g.drawImage(buffImage, 0, 0,  this);
   }

  public void update(Graphics g) {
   g.drawImage(buffImage, 0, 0,  this);
   }

  public void initpaint(Graphics g) {
   try {
    buffImage = this.createImage(500, 500);
    offscreen = buffImage.getGraphics();
    offscreen.setColor(Color.black);
    offscreen.fillRect(0, 0, 500, 500);
    offscreen.setColor(Color.white);
    offscreen.setFont(new Font("Courier", Font.ITALIC, 42));
    offscreen.drawString("Hello World!", 0, 50);
    initDone = true;
    g.drawImage(buffImage,0,0, this);
    }
   catch (Exception e) {
    e.printStackTrace();
    }
   }
  }
*/ 