/*
 * SaviUrlDialog.java
 *
 * Created on 11. z 2006, 11:06
 *
 */

package savi.dialogs;

import java.awt.*;
import java.awt.event.*;

/**
 * Class for making dialogs when user asked for URL.
 * @author Premysl JIRIK
 */
public class SaviUrlDialog extends SaviDialogSkeleton implements ActionListener {
    boolean okPressed = false;
    TextField   fieldURL;
    Button      ok, can;
    
    /**
     * Creates a new instance of SaviUrlDialog, initialize caption.
     * @param caption dialog caption
     */
    public SaviUrlDialog(String caption) {
        super(caption);

        setLayout(new BorderLayout());

        Panel pan;
        pan = new Panel();
        pan.setLayout(new BorderLayout());
        pan.add(new Label("URL:"), BorderLayout.WEST);
        pan.add(fieldURL = new TextField("http://", 40));
        add(pan, BorderLayout.NORTH);
                
        pan = new Panel();
        pan.setLayout(new FlowLayout());
        pan.add(ok = new Button("OK"));
        ok.addActionListener(this); 
        pan.add(can = new Button("Cancel"));
        can.addActionListener(this);
        add(pan, BorderLayout.SOUTH);
    }
    
    /**
     * Return <CODE>true</CODE> if user properly closed dialog (press OK).
     * @return <CODE>true</CODE> if user properly closed dialog (press OK).
     */
    public boolean isOK() {
        return okPressed;
    }
    
    /**
     * Return URL if user properly closed dialog (press OK) else return empty string.
     * @return URL or empty string
     */
    public String getURL() {
        return fieldURL.getText();
    }
    
    /**
     * Actions if user press "close".
     */
    protected void closePressed() {
        okPressed = false;
        super.closeCalled();
    }
    
    /**
     * Show dialog to user.
     * @return <CODE>true</CODE> if user closed dialog properly
     */
    public boolean showForm() {
        distributeKeyListeners();
        validate();
        pack();
        Dimension d = getToolkit().getScreenSize();
        setLocation((d.width - getWidth())/2, (d.height - getHeight())/2);

        setVisible(true);
        return okPressed;
    }
    
    /**
     * Handles the actionPerformed event - "close" or "ok".
     * @param ae the action event
     */
    public void actionPerformed(ActionEvent ae) {
        if(ae.getSource() == ok) {
            okPressed = true;
            setVisible(false);
        } else 
            if(ae.getSource() == can) {
                okPressed = false;
                setVisible(false);
            }
    }        
    
}
