/*
 * Intro.java
 *
 * Created on 1. prosinec 2005, 22:42
 *
 */

package savi.visualization.addons;

/**
 *
 * @author Premysl JIRIK
 */

import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import savi.alg.*;
import savi.visualization.SimAnnealingApplet;
import savi.visualization.support.VisualizationPanel;
import savi.visualization.support.TaskPanel;

/**
 * Visualization add-on with basic information about this applet ("About...")
 * @author Premysl JIRIK
 */
public class Intro extends TaskPanel implements ActionListener {
    /**
     * panel with switchers
     */
    protected Panel                 switcher;
    /**
     * list of help images
     */
    protected ArrayList             images;
    /**
     * switchers
     */
    protected ArrayList             buttons;
    /**
     * index to current page
     */
    protected int                   actualPage = 0;
    
        
    /**
     * Creates a new instance of Intro
     */
    public Intro() {
        super();
        setBackground(Color.darkGray);
        this.setLayout(null);

        images = new ArrayList();
        buttons = new ArrayList();
        switcher = new Panel();
        switcher.setLayout(new GridLayout(1, 1));

        addPage("About this applet", "/res/intro/intro01.gif");
        addPage("Simulated annealing algorithm", "/res/intro/intro02.gif");
        addPage("Applet controls", "/res/intro/intro03.gif");
        
        switcher.setBounds(0, 0, 630, 25);
        switcher.setBackground(Color.WHITE);
        switcher.doLayout();
        
        add(switcher);
    }
    
    /**
     * Return name of the problem.
     * @return name of the problem
     */
    public String getSimulationName() {
        return "A brief intro to this applet";
    }
    
    
    /**
     * Adds new page to list.
     * @param caption page caption
     * @param path path to image resource
     */
    protected void addPage(String caption, String path) {
        Button b;
        Image img = SimAnnealingApplet.getAppletInstance().getImage(getClass().getResource(path));
        if (img == null)
            return;
        
        GridLayout gl = (GridLayout)switcher.getLayout();
        gl.setColumns(gl.getColumns() + 1);

        switcher.add(b = new Button(caption));
        b.addActionListener(this);
        b.setFocusable(false);
        if (buttons.size() == 0) {
            b.setBackground(Color.WHITE);
            b.setForeground(Color.BLACK);
        } else {
            b.setBackground(Color.LIGHT_GRAY);
            b.setForeground(Color.BLACK);
        }
            
        buttons.add(b);
        images.add(img);
    }
    
    /**
     * Not used, ignored.
     * @param s state
     */
    public void setAlgorithm(SimAnnealingSkeleton s) {
    }
    
    /**
     * Update panel (repaint display).
     */
    public void updateScene() {
        this.repaint();
    }

    /**
     * Paints this canvas.
     * @param g the specified Graphics context
     */
    public void paint(Graphics g) {
        g.setColor(Color.WHITE);
        g.fillRect(0, 25, 630, 440);
        if (images.size() > 0)
            g.drawImage((Image)images.get(actualPage), 0, 35, this);            
    }
    /**
     * Updates this canvas.
     * This method is called in response to a call to <CODE>repaint</CODE>.
     * @param g the specified Graphics context
     */
    public void update(Graphics g) {
        this.paint(g);
    }
    
    /**
     * Handles the actionPerformed event - click on a button (page switcher).
     * @param e the action event
     */
    public void actionPerformed(ActionEvent e) {
        int idx = buttons.indexOf(e.getSource());
        if (idx == -1)
            return;
        actualPage = idx;
        
        for (int i = 0; i < buttons.size(); i++) {
            Button b = (Button)buttons.get(i);
            if (i == actualPage) {
                b.setBackground(Color.WHITE);
                b.setForeground(Color.BLACK);
            } else {
                b.setBackground(Color.LIGHT_GRAY);
                b.setForeground(Color.BLACK);
            }
        }
        
        updateScene();
    }
}

