/*
 * StateButton.java
 *
 * Created on 4. leden 2007, 11:38
 *
 */

package savi.visualization.support;

import java.awt.*;

/**
 * This class provide state button. Button can stay in pressed state (something
 * like checkbox).
 * @author Premysl JIRIK
 */
public class StateButton extends Label {
    /**
     * text on button
     */
    protected String text = "";
    /**
     * label for text
     */
    protected Label lblText;
    /**
     * state of button
     */
    protected boolean raised = false;
    
    
    /**
     * Creates a new instance of StateButton, initialize caption.
     * @param label text caption
     */
    public StateButton(String label) {
        super(label, Label.CENTER);
        text = label;
        setBackground(Color.LIGHT_GRAY);
    }

    /**
     * Set button state - raised (<CODE>true</CODE>) / unraised (<CODE>false</CODE>).
     * @param top button state
     */
    public void setRaised(boolean top) {
        if (top)
            setBackground(Color.WHITE);
        else
            setBackground(Color.LIGHT_GRAY);
        raised = top;
        this.repaint();
    }
    /**
     * Return <CODE>true</CODE> if button is raised (pressed).
     * @return <CODE>true</CODE> if button is raised (pressed)
     */
    public boolean isRaised() {
        return raised;
    }
    
    /**
     * Return caption of the button.
     * @return caption of the button
     */
    public String getText() {
        return text;
    }
    
    /**
     * Set caption of the button
     * @param txt caption
     */
    public void setText(String txt) {
        text = txt;
        lblText.setText(text);
        validate();
    }
    
    /**
     * Paints this canvas.
     * @param g the specified Graphics context
     */
    public void paint(Graphics g) {
        super.paint(g);
        if (raised)
            g.setColor(Color.DARK_GRAY);
        else
            g.setColor(Color.WHITE);
        g.drawLine(0, 0, this.getWidth() - 1, 0);
        g.drawLine(0, 0, 0, this.getHeight() - 1);
        
        if (raised)
            g.setColor(Color.LIGHT_GRAY);
        else
            g.setColor(Color.DARK_GRAY);
        g.drawLine(this.getWidth() - 1, 0, this.getWidth() - 1, this.getHeight() - 1);
        g.drawLine(0, this.getHeight() - 1, this.getWidth() - 1, this.getHeight() - 1);
    }
}
